﻿using QQPlus.Framework.SDK;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using Dapper;
using System.Text.RegularExpressions;
using System.Web;
using System.Net;
using System.Web.Script.Serialization;
using QQPlus.Framework;
using QQPlus.Framework.Entities;

namespace QQPlus.Cloud
{
    public class CloudPlugin : Plugin
    {
        public CloudPlugin()
        {
            this.Id = "QQPlus.CloudPlugin";
            this.PluginName = "云端API";
            this.Description = "天气、手机归属地、IP查询、人品预测等几十种常用查询";
            this.PluginId = 160;
        }

        public override bool Start()
        {
            sdk = new QQClientSDK();
            sdk.ReceiveClusterIM += sdk_ReceiveClusterIM;
            sdk.ReceiveNormalIM += sdk_ReceiveNormalIM;
            sdk.ReceiveTempIM += sdk_ReceiveTempIM;
            sdk.ReceiveSubjectIM += sdk_ReceiveSubjectIM;
            return base.Start();
        }

        void sdk_ReceiveSubjectIM(object sender, ReceiveSubjectIMQQEventArgs e)
        {
            var friend = client.GetFriend(e.QQ);
            string result = CallAPI(e.QQ, friend.NickName, e.Message.Trim());
            if (!string.IsNullOrWhiteSpace(result))
            {
                OnLog("给【{0}】发送临时消息：{1}", e.QQ, result);
                client.SendSubjectMessage(e.ClusterId, result);
                e.Cancel = true;
            }
        }

        void sdk_ReceiveTempIM(object sender, ReceiveTempIMQQEventArgs e)
        {
            ClusterConfig cc = null;
            if (e.ExternalId > 10000)
            {
                cc = db.GetClusterConfig(e.ExternalId);
                if (!cc.Status)
                    return;
            }
            var friend = client.GetFriend(e.QQ);
            string result = CallAPI(e.QQ, friend.NickName, e.Message.Trim(), e.ExternalId, cc);
            if (!string.IsNullOrWhiteSpace(result))
            {
                if (e.ExternalId > 10000)
                {
                    long ret = QQPlusDb.UpdateExtcredits(e.ExternalId, e.QQ, cc.ExtcreditsType, cc.ExtcreditsCount);
                    if (ret < 0)
                    {
                        result = cc.Lang2.Replace("{积分类型}", client.Config.ExtcreditsType[(int)cc.ExtcreditsType]);
                        result = result.Replace("{扣除积分}", cc.ExtcreditsCount.ToString() + client.Config.ExtcreditsType[(int)cc.ExtcreditsType]);
                    }
                    else
                    {
                        result = cc.Lang1.Replace("{查询结果}", result);
                        result = result.Replace("{扣除积分}", cc.ExtcreditsCount.ToString() + client.Config.ExtcreditsType[(int)cc.ExtcreditsType]);
                        result = result.Replace("{积分余额}", ret.ToString() + client.Config.ExtcreditsType[(int)cc.ExtcreditsType]);
                    }
                }
                OnLog("给【{0}】发送临时消息：{1}", e.QQ, result);
                client.SendMessage(e.QQ, result);
                e.Cancel = true;
            }
        }

        void sdk_ReceiveNormalIM(object sender, ReceiveNormalIMQQEventArgs e)
        {
            var friend = client.GetFriend(e.QQ);
            string result = CallAPI(e.QQ, friend.NickName, e.Message.Trim());
            if (!string.IsNullOrWhiteSpace(result))
            {
                OnLog("给【{0}】发送消息：{1}", e.QQ, result);
                client.SendMessage(e.QQ, result);
                e.Cancel = true;
            }
        }

        string ReceiveAdminMessage(ClusterConfig cc, Cluster cluster, ClusterMember member, string Message)
        {
            string result = null;
            if (!Message.StartsWith("#"))
                return null;
            if (client.Config.AdminQQs.Contains(member.QQ) || cluster.Creator == member.QQ || member.IsAdmin())
            {
                if ("#开启api".Equals(Message, StringComparison.CurrentCultureIgnoreCase))
                {
                    if (cc.ExternalId == 0 && !client.Config.AdminQQs.Contains(member.QQ))
                        return "管理员禁止查看和修改api设置";
                    else if (cc.Status)
                    {
                        result = "api已经开启，如需查看设置，请回复：#api设置";
                    }
                    else
                    {
                        cc.Status = true;
                        db.SaveClusterConfig(cc);
                        result = "开启api成功，如需查看设置，请回复：#api设置";
                    }
                }
                else if ("#关闭api".Equals(Message, StringComparison.CurrentCultureIgnoreCase))
                {
                    if (cc.ExternalId == 0 && !client.Config.AdminQQs.Contains(member.QQ))
                        return "管理员禁止查看和修改api设置";
                    else if (cc.Status)
                    {
                        cc.Status = false;
                        db.SaveClusterConfig(cc);
                        result = "关闭api成功，如需开启，请回复：#开启api";
                    }
                    else
                    {
                        result = "api已经关闭，如需开启，请回复：#开启api";
                    }
                }
                else if ("#api设置".Equals(Message, StringComparison.CurrentCultureIgnoreCase))
                {
                    if (cc.ExternalId == 0 && !client.Config.AdminQQs.Contains(member.QQ))
                        return "管理员禁止查看和修改api设置";
                    else
                    {
                        return string.Format(@"#api设置
开关={0}
查询扣除积分类型={1}
查询扣除积分数量={2}
查询成功回复={3}
余额不足回复={4}

修改并回复以上内容即可修改设置", cc.Status ? "开" : "关", client.Config.ExtcreditsType[(int)cc.ExtcreditsType]
                   , cc.ExtcreditsCount, cc.Lang1, cc.Lang2);
                    }
                }
                var m = Regex.Match(Message, @"#api设置(.*?)$", RegexOptions.IgnoreCase | RegexOptions.Singleline);
                if (m.Success)
                {
                    if (cc.ExternalId == 0 && !client.Config.AdminQQs.Contains(member.QQ))
                        return "管理员禁止查看和修改api设置";
                    else
                    {
                        m = Regex.Match(Message.Substring(6).Trim(), @"(\S{1,20})\=([\s\S]*?)(?=(\r\n?\S{1,20}\=)|$)");
                        if (m.Success)
                        {
                            while (m.Success)
                            {
                                switch (m.Groups[1].Value)
                                {
                                    case "开关":
                                        cc.Status = m.Groups[2].Value == "开";
                                        break;
                                    case "查询扣除积分类型":
                                        cc.ExtcreditsType = client.Config.ExtcreditsType.IndexOf(m.Groups[2].Value.Trim());
                                        break;
                                    case "查询扣除积分数量":
                                        cc.ExtcreditsCount = long.Parse(m.Groups[2].Value.Trim());
                                        break;
                                    case "查询成功回复":
                                        cc.Lang1 = m.Groups[2].Value;
                                        break;
                                    case "余额不足回复":
                                        cc.Lang2 = m.Groups[2].Value;
                                        break;
                                    default:
                                        return string.Format("发现未知参数：{0}", m.Groups[1].Value);
                                }
                                m = m.NextMatch();
                            }
                            db.SaveClusterConfig(cc);
                        }

                        return string.Format(@"#api设置
开关={0}
查询扣除积分类型={1}
查询扣除积分数量={2}
查询成功回复={3}
余额不足回复={4}

修改并回复以上内容即可修改设置", cc.Status ? "开" : "关", client.Config.ExtcreditsType[(int)cc.ExtcreditsType]
                   , cc.ExtcreditsCount, cc.Lang1, cc.Lang2);
                    }
                }

            }
            return result;
        }

        void sdk_ReceiveClusterIM(object sender, ReceiveClusterIMQQEventArgs e)
        {
            var cc = db.GetClusterConfig(e.Cluster.ExternalId);

            string result = ReceiveAdminMessage(cc, e.Cluster, e.ClusterMember, e.Message);
            if (!string.IsNullOrWhiteSpace(result))
            {
                OnLog("给【{0}】发送QQ群消息：{1}", e.Cluster, result);
                client.SendClusterMessage(e.Cluster, e.ClusterMember, result, null, false, false);
                e.Cancel = true;
                return;
            }

            if (!cc.Status)
                return;

            result = CallAPI(e.ClusterMember.QQ, e.ClusterMember.Nick, e.Message.Trim(), e.Cluster.ExternalId, cc);
            if (!string.IsNullOrWhiteSpace(result))
            {
                OnLog("给【{0}】发送QQ群消息：{1}", e.Cluster.ExternalId, result);
                client.SendClusterMessage(e.Cluster, e.ClusterMember, result);
                e.Cancel = true;
                return;
            }
        }

        public override bool ShowForm()
        {
            new FormMain(this).ShowDialog();
            return base.ShowForm();
        }

        string CallAPI(uint QQ, string nick, string msg, uint ExternalId = 0, ClusterConfig cc = null)
        {
            if (string.IsNullOrWhiteSpace(msg) || !msg.StartsWith("#"))
                return null;

            msg = msg.Trim("#".ToCharArray());
            string result = string.Empty;

            try
            {
                foreach (var api in db.GetAllAPI())
                {
                    if (!api.Status)
                        continue;

                    var m = Regex.Match(msg, api.Command, RegexOptions.IgnoreCase);
                    if (m.Success)
                    {
                        long ret = 0;
                        if (ExternalId > 10000 && cc != null)
                        {
                            ret = QQPlusDb.UpdateExtcredits(ExternalId, QQ, cc.ExtcreditsType, -1 * cc.ExtcreditsCount);
                            if (ret < 0)
                            {
                                result = cc.Lang2.Replace("{积分类型}", client.Config.ExtcreditsType[(int)cc.ExtcreditsType]);
                                result = result.Replace("{扣除积分}", cc.ExtcreditsCount.ToString() + client.Config.ExtcreditsType[(int)cc.ExtcreditsType]);
                                return result;
                            }
                        }

                        string msg1 = string.Empty;
                        if (m.Groups["msg1"] != null)
                            msg1 = m.Groups["msg1"].Value.Trim();

                        OnLog("调用云端API：{0}", api.Url);
                        Encoding encoding = Encoding.GetEncoding(api.Encode);
                        string url = api.Url
                            .Replace("[key]", api.Key)
                            .Replace("[msg]", HttpUtility.UrlEncode(msg, encoding))
                            .Replace("[msg1]", HttpUtility.UrlEncode(msg1, encoding))
                            .Replace("[nick]", HttpUtility.UrlEncode(nick, encoding))
                            .Replace("[qq]", QQ.ToString())
                            .Replace("[robot]", client.User.QQ.ToString());

                        byte[] data = new WebClient().DownloadData(url);
                        result = encoding.GetString(data);
                        OnLog("收到返回：{0}", result);
                        switch (api.Protocol)
                        {
                            case "TEXT":
                                break;
                            case "JSON":
                                result = new JavaScriptSerializer().Deserialize<object>(result).ToString();
                                break;
                            case "XML":
                                result = Regex.Replace(result, "<[^>]+/?>", "");
                                break;
                            default:
                                return "编码错误";
                        }
                        result = cc.Lang1.Replace("{查询结果}", result);
                        result = result.Replace("{扣除积分}", cc.ExtcreditsCount.ToString() + client.Config.ExtcreditsType[(int)cc.ExtcreditsType]);
                        result = result.Replace("{积分余额}", ret.ToString() + client.Config.ExtcreditsType[(int)cc.ExtcreditsType]);
                    }
                }
            }
            catch (Exception ex)
            {
                OnLog("云端API调用失败：{0}\r\n{1}", ex.Message, ex.StackTrace);
            }

            return result;
        }
    }
}
