﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Net;
using System.Text;
using System.Windows.Forms;
using Dapper;
using System.Web.Script.Serialization;
using System.IO;
using QQPlus.Framework.Utils;

namespace QQPlus.Cloud
{
    public partial class FormMain : QQPlus.Forms.FormBase
    {
        public FormMain()
        {
            InitializeComponent();
        }

        public FormMain(CloudPlugin plugin)
            : this()
        {
            // TODO: Complete member initialization
            this.plugin = plugin;
        }

        private void FormMain_Load(object sender, EventArgs e)
        {
            cmbProtocol.SelectedIndex = 0;

            ListViewItem item = null;
            db.GetAllAPI().ForEach(api =>
                {
                    item = new ListViewItem(new string[] { api.Id.ToString(), api.Command, api.Status ? "开" : "关", api.Encode, api.Comments, api.Url, api.Protocol, api.Key });
                    item.Tag = api;
                    lvAPI.Items.Add(item);
                }
            );
            var cc = db.GetClusterConfig(0);
            item = new ListViewItem(new string[] { "0", "全局设置", cc.Status ? "开" : "关" });
            item.Tag = cc;
            lvCluster.Items.Add(item);

            cc = db.GetClusterConfig(1);
            item = new ListViewItem(new string[] { "1", "默认设置", cc.Status ? "开" : "关" });
            item.Tag = cc;
            lvCluster.Items.Add(item);

            plugin.client.User.Clusters.ForEach(t =>
                {
                    cc = db.GetClusterConfig(t.ExternalId);
                    item = new ListViewItem(new string[] { t.ExternalId.ToString(), t.Name, cc.Status ? "开" : "关" });
                    item.Tag = cc;
                    lvCluster.Items.Add(item);
                }
            );
        }

        private void 删除ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            List<long> ids = new List<long>();
            while (lvAPI.SelectedItems.Count > 0)
            {
                var item = lvAPI.SelectedItems[lvAPI.SelectedItems.Count - 1];
                ids.Add(long.Parse(item.SubItems[0].Text));
                lvAPI.Items.Remove(item);
            }

            if (ids.Count > 0)
            {
                db.DeleteAPI(ids);
            }
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            var api = new API() { Command = txtCommand.Text, Comments = txtComments.Text, Encode = cmbEncode.Text, Key = txtKey.Text, Protocol = cmbProtocol.Text, Status = chkStatus.Checked, Url = txtUrl.Text };
            db.SaveAPI(api);
            var item = new ListViewItem(new string[] { api.Id.ToString(), api.Command, api.Status ? "开" : "关", api.Encode, api.Comments, api.Url, api.Protocol, api.Key });
            item.Tag = api;
            lvAPI.Items.Add(item);

            txtUrl.Clear();
            txtCommand.Clear();
            txtComments.Clear();
            btnEdit.Tag = null;
            btnEdit.Enabled = false;
        }

        private CloudPlugin plugin;
        private void lvAPI_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (lvAPI.SelectedItems.Count > 0)
            {
                var item = lvAPI.SelectedItems[0];
                var api = item.Tag as API;
                btnEdit.Enabled = true;
                btnEdit.Tag = item;
                txtCommand.Text = item.SubItems[1].Text;
                chkStatus.Checked = (item.SubItems[2].Text == "开");
                cmbEncode.Text = item.SubItems[3].Text;
                txtComments.Text = item.SubItems[4].Text;
                txtUrl.Text = item.SubItems[5].Text;
                cmbProtocol.Text = item.SubItems[6].Text;
                txtKey.Text = item.SubItems[7].Text;
            }
            else
                btnEdit.Enabled = false;
        }

        private void btnEdit_Click(object sender, EventArgs e)
        {
            var item = btnEdit.Tag as ListViewItem;
            var api = item.Tag as API;
            api.Command = txtCommand.Text;
            api.Comments = txtComments.Text;
            api.Encode = cmbEncode.Text;
            api.Key = txtKey.Text;
            api.Protocol = cmbProtocol.Text;
            api.Status = chkStatus.Checked;
            api.Url = txtUrl.Text;
            db.SaveAPI(api);

            UpdateItem(item);


            txtUrl.Clear();
            txtCommand.Clear();
            txtComments.Clear();
            btnEdit.Tag = null;
            btnEdit.Enabled = false;
        }

        private void UpdateItem(ListViewItem item)
        {
            var api = item.Tag as API;
            item.SubItems[1].Text = api.Command;
            item.SubItems[2].Text = api.Status ? "开" : "关";
            item.SubItems[3].Text = api.Encode;
            item.SubItems[4].Text = api.Comments;
            item.SubItems[5].Text = api.Url;
            item.SubItems[6].Text = api.Protocol;
            item.SubItems[7].Text = api.Key;
        }

        private void 开启关闭ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (lvAPI.SelectedItems.Count > 0)
            {
                List<long> ids = new List<long>();
                for (int i = 0; i < lvAPI.SelectedItems.Count; i++)
                {
                    var item = lvAPI.SelectedItems[i];
                    var api = item.Tag as API;
                    api.Status = !api.Status;
                    db.SaveAPI(api);
                    UpdateItem(item);
                }
            }
        }

        private void 导入ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("密钥不会被导入，请先填写密钥，现在开始导入吗？", "提示", MessageBoxButtons.OKCancel) == System.Windows.Forms.DialogResult.OK)
            {
                var dlg = new OpenFileDialog() { Filter = "文本文档(*.txt)|*.txt" };
                if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    string txt = File.ReadAllText(dlg.FileName, Encoding.Default);
                    var apis1 = new JavaScriptSerializer().Deserialize<API[]>(txt);
                    apis1.ToList().ForEach(api =>
                    {
                        if (db.GetAllAPI().FirstOrDefault(a1 => a1.Url == api.Url) == null)
                        {
                            api.Key = txtKey.Text;
                            db.SaveAPI(api);

                            var item = new ListViewItem(new string[] { api.Id.ToString(), api.Command, api.Status ? "开" : "关", api.Encode, api.Comments, api.Url, api.Protocol, api.Key });
                            item.Tag = api;
                            lvAPI.Items.Add(item);
                        }
                    }
                    );
                }
            }
        }

        private void 导出ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("密钥不会被导出，现在开始导出吗？", "提示", MessageBoxButtons.OKCancel) == System.Windows.Forms.DialogResult.OK)
            {
                SaveFileDialog dlg = new SaveFileDialog() { Filter = "文本文档(*.txt)|*.txt" };
                if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    List<API> apis = new List<API>();
                    for (int i = 0; i < lvAPI.Items.Count; i++)
                    {
                        var api = lvAPI.Items[i].Tag as API;
                        api.Key = string.Empty;
                        apis.Add(api);
                    }
                    string txt = new JavaScriptSerializer().Serialize(apis);
                    File.WriteAllText(dlg.FileName, txt, Encoding.Default);
                    System.Diagnostics.Process.Start("Explorer", "/select," + dlg.FileName);
                }
            }
        }

        private void lvAPI_ColumnClick(object sender, ColumnClickEventArgs e)
        {
            if (this.lvAPI.Columns[e.Column].Tag == null)
                this.lvAPI.Columns[e.Column].Tag = true;
            bool flag = (bool)this.lvAPI.Columns[e.Column].Tag;
            if (flag)
                this.lvAPI.Columns[e.Column].Tag = false;
            else
                this.lvAPI.Columns[e.Column].Tag = true;
            switch (e.Column)
            {
                case 0:
                    this.lvAPI.ListViewItemSorter = new ListViewSort(e.Column, this.lvAPI.Columns[e.Column].Tag, SortType.Value);
                    break;
                default:
                    this.lvAPI.ListViewItemSorter = new ListViewSort(e.Column, this.lvAPI.Columns[e.Column].Tag, SortType.String);
                    break;
            }
            this.lvAPI.Sort();//对列表进行自定义排序  

        }

        private void lvCluster_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if (lvCluster.SelectedItems.Count > 0)
            {
                var item = lvCluster.SelectedItems[0];
                var cc = item.Tag as ClusterConfig;
                if (new FormConfig(plugin, cc).ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    item.SubItems[2].Text = cc.Status ? "开" : "关";
                }
            }
        }
    }
}
