﻿using Dapper;
using QQPlus.Framework.Utils;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace QQPlus.Cloud
{
    public class ClusterConfig
    {
        [Key]
        public long Id { get; set; }

        public long ExternalId { get; set; }
        /// <summary>
        /// 开关
        /// </summary>
        public bool Status { get; set; }
        /// <summary>
        /// 积分类型
        /// </summary>
        public long ExtcreditsType { get; set; }
        /// <summary>
        /// 积分数量
        /// </summary>
        public long ExtcreditsCount { get; set; }

        /// <summary>
        /// 查询成功
        /// </summary>
        public string Lang1 { get; set; }

        public ClusterConfig()
        {
            Status = true;
            ExtcreditsType = 0;
            ExtcreditsCount = 5;
            Lang1 = @"[@{QQ}]
{查询结果}
本次查询扣除{扣除积分}，余额{积分余额}";
            this.Lang2 = "[@{QQ}]，{积分类型}不足，查询需要消耗{扣除积分}";
        }

        public ClusterConfig Clone()  //对外提供一个创建自身的浅表副本的能力
        {
            return (ClusterConfig)this.MemberwiseClone();
        }

        /// <summary>
        /// 余额不足
        /// </summary>
        public string Lang2 { get; set; }
    }

    public class API
    {
        [Key]
        public long Id { get; set; }
        public string Command { get; set; }
        public bool Status { get; set; }
        public string Encode { get; set; }
        public string Comments { get; set; }
        public string Url { get; set; }
        public string Protocol { get; set; }
        public string Key { get; set; }
    }

    public class db : DbBase
    {
        internal static readonly object LockDb = new object();
        static string conn = string.Format("Data Source={0};Pooling=true;FailIfMissing=false", Util.MapFile("plugin\\QQPlus.Cloud.db"));
        static List<API> apis = null;
        static List<ClusterConfig> ccs = null;

        public db()
            : base(conn)
        {
        }

        static db()
        {
            lock (LockDb)
            {
                using (db d = new db())
                {
                    Version v = new Version();
                    if (!d.Conn.TableExists("_history"))
                    {
                        //初始化
                        d.Conn.Execute(@"CREATE TABLE [_history] (
[Id] INTEGER  PRIMARY KEY NOT NULL,
[Ver] NVARCHAR(50)  NULL
);
INSERT INTO [_history] (ver) values('2.0.1405.6');

CREATE TABLE [API] (
[Id] INTEGER PRIMARY KEY NOT NULL,
[Command] NVARCHAR(500) NULL,
[Status] BOOLEAN DEFAULT '1' NOT NULL,
[Encode] NVARCHAR(500) NULL,
[Comments] NVARCHAR(500) NULL,
[Url] NVARCHAR(500) NULL,
[Protocol] NVARCHAR(500) NULL,
[Key] NVARCHAR(500) NULL
);

create table [ClusterConfig] (
[Id] integer primary key not null,
[ExternalId] integer default '0' not null,
[Status] boolean default '1' not null,
[ExtcreditsType] integer default '0' not null,
[ExtcreditsCount] integer default '5' not null,
[Lang1] text,
[Lang2] text
);
");
                        d.Conn.InsertMany(new ClusterConfig[] {
                            new ClusterConfig() { ExternalId = 0, Status = false }
                            , new ClusterConfig() { ExternalId = 1, Status = true } 
                        });
                    }

                    apis = d.Conn.GetAll<API>().ToList();
                    ccs = d.Conn.GetAll<ClusterConfig>().ToList();
                }
            }
        }

        internal static List<API> GetAllAPI()
        {
            return apis;
        }

        internal static ClusterConfig GetClusterConfig(long ExternalId)
        {
            try
            {
                lock (LockDb)
                {
                    if (ExternalId < 10000)
                        return ccs.FirstOrDefault(t => t.ExternalId == ExternalId);

                    var cc = ccs.FirstOrDefault(cc1 => cc1.ExternalId == 0);
                    if (cc != null && cc.Status)
                        return cc;

                    cc = ccs.FirstOrDefault(cc1 => cc1.ExternalId == ExternalId);
                    if (cc == null)//复制默认设置
                    {
                        var cc0 = ccs.FirstOrDefault(cc1 => cc1.ExternalId == 1);
                        cc = cc0.Clone();
                        cc.Id = 0;
                        cc.ExternalId = ExternalId;
                        SaveClusterConfig(cc);

                    }
                    return cc;
                }
            }
            catch (Exception ex)
            {
                QQLog.Write("群{2}配置初始化失败：{0}\n{1}", ex.Message, ex.StackTrace, ExternalId);
            }

            return null;
        }

        internal static void SaveClusterConfig(ClusterConfig cc)
        {
            lock (LockDb)
            {
                using (db d = new db())
                {
                    if (cc.Id > 0)
                        d.Conn.Update(cc);
                    else
                    {
                        cc.Id = d.Conn.Insert(cc);
                        ccs.Add(cc);
                    }
                }
            }
        }

        internal static void SaveAPI(API api)
        {
            lock (LockDb)
            {
                using (db d = new db())
                {
                    if (api.Id > 0)
                        d.Conn.Update(api);
                    else
                    {
                        api.Id = d.Conn.Insert(api);
                        apis.Add(api);
                    }
                }
            }
        }

        internal static int DeleteAPI(List<long> ids)
        {
            lock (LockDb)
            {
                using (db d = new db())
                {
                    apis.RemoveAll(t => ids.Contains(t.Id));
                    return d.Conn.DeleteMany<API>(ids);
                }
            }
        }
    }
}
