﻿using QQPlus.Framework.SDK;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using Dapper;
using QQPlus.Framework.Entities;
using System.Text.RegularExpressions;
using QQPlus.Framework.Utils;
using QQPlus.Framework;

namespace QQPlus.ClusterJoin
{
    public class ClusterJoinPlugin : Plugin
    {
        public ClusterJoinPlugin()
        {
            this.Id = "QQPlus.ClusterJoin";
            this.PluginName = "群互联";
            this.Description = "A群中的消息转发到B群中";
            this.PluginId = 161;

            var db = Path.Combine(PluginPath, "QQPlus.ClusterJoin.db");
            DbBase.DataSource = string.Format("Data Source={0};Pooling=true;FailIfMissing=false", db);
        }

        public override bool Install()
        {
            lock (DbBase.LockDb)
            {
                using (DbBase db = new DbBase())
                {
                    Version v = new Version();
                    if (!db.Conn.TableExists("_history"))
                    {
                        //初始化
                        db.Conn.Execute(@"CREATE TABLE [_history] (
[Id] INTEGER  PRIMARY KEY NOT NULL,
[Ver] NVARCHAR(50)  NULL
);
INSERT INTO [_history] (ver) values('2.0.1405.6');

CREATE TABLE [Config](
[Id] INTEGER PRIMARY KEY NOT NULL,
[Content] TEXT  NULL
);
INSERT INTO [Config] (Content) values('');

CREATE TABLE [ClusterJoin] (
[Id] INTEGER PRIMARY KEY NOT NULL,
[ExternalId1] INTEGER NOT NULL,
[ExternalId2] INTEGER NOT NULL,
[Format] TEXT  NULL,
[QQ] TEXT  NULL
);");
                    }
                }
            }
            return base.Install();
        }

        public override bool ShowForm()
        {
            new FormMain(this).ShowDialog();
            LoadJoin();
            return base.ShowForm();
        }

        public ClusterJoinConfig config = null;
        public override bool Start()
        {
            LoadJoin();

            sdk = new QQClientSDK();
            sdk.ReceiveClusterIM += sdk_ReceiveClusterIM;
            sdk.ReceiveNormalIM += sdk_ReceiveNormalIM;
            return base.Start();
        }

        void sdk_ReceiveNormalIM(object sender, ReceiveNormalIMQQEventArgs e)
        {
            if (e.Message.StartsWith("#") && client.Config.AdminQQs.Contains(e.QQ))
            {
                var result = ReceiveAdminMessage(e.Message.TrimStart("#".ToCharArray()));
                if (!string.IsNullOrWhiteSpace(result))
                {
                    client.SendMessage(e.QQ, result);
                    e.Cancel = true;
                }
            }
        }

        private string ReceiveAdminMessage(string Message)
        {
            string help = @"
其他可用指令如下：
#查群互联
#删群互联+群号码，例如：#删群互联 111111
#双向互联+群号码+转发格式，多个号码使用逗号分隔，例如：#双向互联 11111,22222,33333 {昵称}/{群名} {消息内容}
#单向互联+群号码+转发格式，只能进行2个群的单向互联，例如：#单向互联 111111,222222 {昵称}/{群名} {消息内容}
转发内容支持变量：{QQ}、{昵称}、{名片}、{群名}、{群号}、{时间}、{消息内容}
";

            var msg = Message.TrimStart("#".ToCharArray());
            if (msg == "查群互联")
            {
                StringBuilder sb = new StringBuilder("互联设置如下：\n");
                cjs.ForEach(c =>
                    {
                        sb.AppendFormat(@"{0}=>{1}
{2}
", c.ExternalId1, c.ExternalId2, c.Format);
                    }
                );

                sb.Append(help);
                return sb.ToString();
            }
            var m = Regex.Match(Message, @"^双向互联 ([\d，,]+) (.*?)$", RegexOptions.IgnoreCase);
            if (m.Success)
            {
                List<string> qqs = m.Groups[1].Value.Split(",，".ToCharArray(), StringSplitOptions.RemoveEmptyEntries).ToList();
                if (qqs.Count < 2)
                {
                    return "群互联至少是2个群\n" + help;
                }
                string format = m.Groups[2].Value;

                for (int i = 0; i < qqs.Count; i++)
                {
                    for (int j = 0; j < qqs.Count; j++)
                    {
                        if (j == i)
                            continue;

                        uint ExternalId = uint.Parse(qqs[i]);
                        uint ExternalId1 = uint.Parse(qqs[j]);
                        var cj = cjs.FirstOrDefault(c => c.ExternalId1 == ExternalId && c.ExternalId2 == ExternalId1);
                        using (DbBase db = new DbBase())
                        {
                            if (cj == null)
                            {
                                cj = new ClusterJoin() { ExternalId1 = ExternalId, ExternalId2 = ExternalId1, Format = format };
                                cj.Id = db.Conn.Insert(cj);
                                cjs.Add(cj);
                            }
                            else
                            {
                                cj.Format = format;
                                db.Conn.Update(cj);
                            }
                        }
                    }
                }
                return "操作成功\n" + help;
            }
            m = Regex.Match(Message, @"^单向互联 ([\d,，]+) (.*?)$", RegexOptions.IgnoreCase);
            if (m.Success)
            {
                List<string> qqs = m.Groups[1].Value.Split(",，".ToCharArray(), StringSplitOptions.RemoveEmptyEntries).ToList();
                if (qqs.Count < 2)
                {
                    return "群互联至少是2个群\n" + help;
                }

                string format = m.Groups[2].Value;

                for (int i = 0; i < qqs.Count; i += 2)
                {
                    uint ExternalId = uint.Parse(qqs[i]);
                    uint ExternalId1 = uint.Parse(qqs[i + 1]);
                    var cj = cjs.FirstOrDefault(c => c.ExternalId1 == ExternalId && c.ExternalId2 == ExternalId1);
                    using (DbBase db = new DbBase())
                    {
                        if (cj == null)
                        {
                            cj = new ClusterJoin() { ExternalId1 = ExternalId, ExternalId2 = ExternalId1, Format = format };
                            cj.Id = db.Conn.Insert(cj);
                            cjs.Add(cj);
                        }
                        else
                        {
                            cj.Format = format;
                            db.Conn.Update(cj);
                        }
                    }
                }
                return "操作成功\n" + help;
            }
            m = Regex.Match(Message, @"^删群互联 ([\d，,]+)$", RegexOptions.IgnoreCase);
            if (m.Success)
            {
                int count = 0;
                m.Groups[1].Value.Split(",，".ToCharArray(), StringSplitOptions.RemoveEmptyEntries).ToList().ForEach(q =>
                {
                    uint ExternalId = uint.Parse(q);
                    using (DbBase db = new DbBase())
                    {
                        db.Conn.Execute("DELETE FROM ClusterJoin WHERE ExternalId1=@ExternalId OR ExternalId2=@ExternalId", new { ExternalId = (long)ExternalId });
                    }
                    count += cjs.RemoveAll(c => c.ExternalId1 == ExternalId || c.ExternalId2 == ExternalId);
                }
                );
                if (count > 0)
                {
                    return "操作成功，删除互联" + count.ToString() + "个\n" + help;
                }
                else
                    return "没有找到匹配设置\n" + help;
            }
            return null;
        }

        public List<ClusterJoin> cjs = null;
        public void LoadJoin()
        {
            using (DbBase db = new DbBase())
            {
                cjs = db.Conn.GetAll<ClusterJoin>().ToList();
            }
            cjs.ForEach(c => c.QQs = Util.StrToList(c.QQ));
        }

        void sdk_ReceiveClusterIM(object sender, ReceiveClusterIMQQEventArgs e)
        {
            var cj = cjs.Where(c => c.ExternalId1 == e.Cluster.ExternalId).ToList();

            cj.ForEach(c =>
                {
                    if (c.QQs.Count == 0 || c.QQs.Contains(e.ClusterMember.QQ))
                    {
                        string msg = ReplaceParams(c.Format, e.Cluster, e.ClusterMember, e.Message);
                        client.SendClusterMessage((uint)c.ExternalId2, msg, e.Style);
                    }
                }
            );
        }

        string ReplaceParams(string Message, Cluster c = null, ClusterMember cm = null, string msg = "")
        {
            try
            {
                if (string.IsNullOrWhiteSpace(Message))
                    return string.Empty;

                if (c != null)
                    Message = Message.Replace("{群名}", c.Name).Replace("{群号}", c.ExternalId.ToString());
                if (cm != null)
                    Message = Message.Replace("{QQ}", cm.QQ.ToString()).Replace("{昵称}", cm.Nick).Replace("{名片}", cm.Card);
                Message = Message.Replace("{消息内容}", msg);
                return Message;
            }
            catch (Exception ex)
            {
                OnLog("替换变量失败：{0}\n{1}", ex.Message, ex.StackTrace);
            }

            return null;
        }
    }
}
