﻿using QQPlus.Framework.Utils;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Dapper;

namespace QQPlus.ClusterJoin
{
    public partial class FormMain : QQPlus.Forms.FormBase
    {
        ClusterJoinPlugin plugin = null;
        public FormMain(ClusterJoinPlugin plugin)
        {
            this.plugin = plugin;
            InitializeComponent();
        }

        private void FormMain_Load(object sender, EventArgs e)
        {
            lvJoin.Items.Clear();
            plugin.cjs.ForEach(c =>
                {
                    lvJoin.Items.Add(new ListViewItem(new string[] { c.Id.ToString(), c.ExternalId1.ToString(), c.ExternalId2.ToString(), c.Format, c.QQ }));
                }
            );
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            using (DbBase db = new DbBase())
            {
                uint ExternalId1 = uint.Parse(txtExternalId1.Text.Trim());
                uint ExternalId2 = uint.Parse(txtExternalId2.Text.Trim());
                var cj = plugin.cjs.FirstOrDefault(c => c.ExternalId1 == ExternalId1 && c.ExternalId2 == ExternalId2);
                if (cj == null)
                {
                    cj = new ClusterJoin() { ExternalId1 = ExternalId1, ExternalId2 = ExternalId2, Format = txtFormat.Text, QQ = txtQQ.Text };
                    db.Conn.Insert(cj);
                }
                else
                {
                    cj.Format = txtFormat.Text;
                    cj.QQ = txtQQ.Text;
                    db.Conn.Update(cj);
                }
            }
            plugin.LoadJoin();
            FormMain_Load(null, null);
        }

        private void lvJoin_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (lvJoin.SelectedItems.Count > 0)
            {
                var item = lvJoin.SelectedItems[0];
                txtExternalId1.Text = item.SubItems[1].Text;
                txtExternalId2.Text = item.SubItems[2].Text;
                txtFormat.Text = item.SubItems[3].Text;
                txtQQ.Text = item.SubItems[4].Text;
            }
        }

        private void 删除ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            List<long> ids = new List<long>();
            while (lvJoin.SelectedItems.Count > 0)
            {
                var item = lvJoin.SelectedItems[lvJoin.SelectedItems.Count - 1];
                ids.Add(long.Parse(item.SubItems[0].Text));
                lvJoin.Items.Remove(item);
            }

            if (ids.Count > 0)
            {
                using (DbBase db = new DbBase())
                {
                    db.Conn.DeleteMany<ClusterJoin>(ids);
                }
                plugin.LoadJoin();
            }
        }

        private void lvJoin_ColumnClick(object sender, ColumnClickEventArgs e)
        {
            if (this.lvJoin.Columns[e.Column].Tag == null)
                this.lvJoin.Columns[e.Column].Tag = true;
            bool flag = (bool)this.lvJoin.Columns[e.Column].Tag;
            if (flag)
                this.lvJoin.Columns[e.Column].Tag = false;
            else
                this.lvJoin.Columns[e.Column].Tag = true;
            switch (e.Column)
            {
                case 0:
                case 1:
                case 2:
                    this.lvJoin.ListViewItemSorter = new ListViewSort(e.Column, this.lvJoin.Columns[e.Column].Tag, SortType.Value);
                    break;
                default:
                    this.lvJoin.ListViewItemSorter = new ListViewSort(e.Column, this.lvJoin.Columns[e.Column].Tag, SortType.String);
                    break;
            }
            this.lvJoin.Sort();//对列表进行自定义排序  

        }
    }
}
