﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Dapper;
using QQPlus.Framework;

namespace QQPlus.MessageBox
{
    public partial class FormMain : QQPlus.Forms.FormBase
    {
        public FormMain()
        {
            InitializeComponent();
        }

        private void FormMain_Load(object sender, EventArgs e)
        {
            HashSet<long> qqs = new HashSet<long>();
            using (DbBase db = new DbBase())
            {
                var msgs = db.Conn.Query<Messages>("select * from Messages where Type=0 order by QQ").ToList();
                foreach (Messages msg in msgs)
                {
                    if (qqs.Contains(msg.QQ))
                        continue;

                    qqs.Add(msg.QQ);
                    TreeNode node = new TreeNode(string.Format("{0}({1})", msg.QQ, msg.NickName));
                    node.Tag = msg;
                    treeView1.Nodes[0].Nodes.Add(node);
                }

                msgs = db.Conn.Query<Messages>("select * from Messages where Type=1 order by QQ").ToList();
                qqs.Clear();
                foreach (Messages msg in msgs)
                {
                    if (qqs.Contains(msg.ExternalId))
                        continue;

                    qqs.Add(msg.ExternalId);
                    TreeNode node = new TreeNode(string.Format("{0}({1})", msg.ExternalId, msg.ClusterName));
                    node.Tag = msg;
                    treeView1.Nodes[1].Nodes.Add(node);
                }

                msgs = db.Conn.Query<Messages>("select * from Messages where Type=2 order by QQ").ToList();
                qqs.Clear();
                foreach (Messages msg in msgs)
                {
                    if (qqs.Contains(msg.QQ))
                        continue;

                    qqs.Add(msg.QQ);
                    TreeNode node = new TreeNode(string.Format("{0}({1})-{2}", msg.QQ, msg.NickName, msg.ClusterName));
                    node.Tag = msg;
                    treeView1.Nodes[2].Nodes.Add(node);
                }
                msgs = db.Conn.Query<Messages>("select * from Messages where Type=3 order by QQ").ToList();
                qqs.Clear();
                foreach (Messages msg in msgs)
                {
                    if (qqs.Contains(msg.QQ))
                        continue;

                    qqs.Add(msg.QQ);
                    TreeNode node = new TreeNode(string.Format("{0}({1})", msg.ExternalId, msg.ClusterName));
                    node.Tag = msg;
                    treeView1.Nodes[3].Nodes.Add(node);
                }
            }
        }

        private void treeView1_AfterSelect(object sender, TreeViewEventArgs e)
        {
            rtxtMsgList.ResetText();
            if (e.Node.Tag != null)
            {
                using (DbBase db = new DbBase())
                {

                    Messages msg = e.Node.Tag as Messages;
                    this.rtxtMsgList.Tag = msg;

                    List<Messages> msgs = null;
                    if (msg.ExternalId > 1000)
                        msgs = db.Conn.Query<Messages>("select * from Messages where ExternalId=@ExternalId order by Id", new { ExternalId = msg.ExternalId }).ToList();
                    else if (msg.ExternalId == 0)
                        msgs = db.Conn.Query<Messages>("select * from Messages where ExternalId=0 and QQ=@QQ order by Id", new { QQ = msg.QQ }).ToList();
                    else if (msg.ExternalId == 1)
                        msgs = db.Conn.Query<Messages>("select * from Messages where ExternalId=1 and QQ=@QQ order by Id", new { QQ = msg.QQ }).ToList();
                    else
                        msgs = db.Conn.Query<Messages>("select * from Messages order by Id").ToList();

                    foreach (Messages m in msgs)
                    {
                        InsertText(m);
                    }
                }
            }
        }

        private void InsertText(Messages m)
        {
            try
            {
                QQPlus.Framework.Entities.FontStyle style = new QQPlus.Framework.Entities.FontStyle(m.FontStyle);
                if (style.FontSize < 9)
                    style.FontSize = 9;
                Font f = new Font(style.FontName, style.FontSize, (FontStyle)(int)style.FontFlag);
                InsertText(new Font("宋体", 9, FontStyle.Regular), Color.Green, "{0}({1}) {2}\n", m.QQ, m.NickName, m.ReceiveTime);
                InsertText(f, style.Color, "{0}\n", m.Message);
            }
            catch (Exception ex)
            { 
            }
        }

        private void InsertText(Font font, Color color, string format, params object[] args)
        {
            format = string.Format(format, args);
            int start = this.rtxtMsgList.SelectionStart + this.rtxtMsgList.SelectionLength;

            this.rtxtMsgList.SelectedText = this.rtxtMsgList.SelectedText.Insert(this.rtxtMsgList.SelectionLength, format);
            this.rtxtMsgList.Select(start, format.Length);
            this.rtxtMsgList.SelectionFont = font;
            this.rtxtMsgList.SelectionColor = color;
            //this.rtxtMsgList.SelectionLength = 0;
        }

        private void 删除ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (rtxtMsgList.Tag != null)
            {
                using (DbBase db = new DbBase())
                {
                    Messages msg = rtxtMsgList.Tag as Messages;
                    if (msg.ExternalId > 1000)
                        db.Conn.Execute("delete from Messages where ExternalId=@ExternalId", new { ExternalId = msg.ExternalId });
                    else if (msg.ExternalId == 0)
                        db.Conn.Execute("delete from Messages where ExternalId=0 and QQ=@QQ", new { QQ = msg.QQ });
                    else if (msg.ExternalId == 1)
                        db.Conn.Execute("delete from Messages where ExternalId=1 and QQ=@QQ", new { QQ = msg.QQ });
                }
                if (treeView1.SelectedNode != null)
                    treeView1.SelectedNode.Remove();

                System.Windows.Forms.MessageBox.Show("删除完成");
            }
        }

        private void 导出ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (rtxtMsgList.Tag != null)
            {
                SaveFileDialog dlg = new SaveFileDialog() { Filter = "RTF文档(*.rtf)|*.rtf|文本文档(*.txt)|*.txt" };
                if (dlg.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                {
                    if (dlg.FileName.EndsWith(".rtf", StringComparison.CurrentCultureIgnoreCase))
                        rtxtMsgList.SaveFile(dlg.FileName, RichTextBoxStreamType.RichText);
                    else if (dlg.FileName.EndsWith(".txt", StringComparison.CurrentCultureIgnoreCase))
                        rtxtMsgList.SaveFile(dlg.FileName, RichTextBoxStreamType.PlainText);

                    System.Windows.Forms.MessageBox.Show("保存完成");
                    System.Diagnostics.Process.Start("Explorer", "/select," + dlg.FileName);
                }
            }
        }
    }
}
