﻿using QQPlus.Framework.SDK;
using QQPlus.Framework.Utils;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Dapper;
using System.IO;

namespace QQPlus.MessageBox
{
    public class MessageBoxPlugin : Plugin
    {
        public MessageBoxPlugin()
        {
            this.Id = "QQPlus.MessageBoxPlugin";
            this.PluginName = "消息盒子";
            this.Description = "记录QQ消息和群消息";
            this.PluginId = 165;

            var db = Path.Combine(PluginPath, "QQPlus.MessageBox.db");
            DbBase.DataSource = string.Format("Data Source={0};Pooling=true;FailIfMissing=false", db);
        }

        public override bool Install()
        {
            lock (DbBase.LockDb)
            {
                using (DbBase db = new DbBase())
                {
                    Version v = new Version();
                    if (!db.Conn.TableExists("_history"))
                    {
                        //初始化
                        db.Conn.Execute(@"CREATE TABLE [_history] (
[Id] INTEGER  PRIMARY KEY NOT NULL,
[Ver] NVARCHAR(50)  NULL
);
INSERT INTO [_history] (ver) values('2.0.1405.6');

CREATE TABLE [Messages] (
[Id] INTEGER PRIMARY KEY NOT NULL,
[Type] INTEGER NOT NULL,
[ExternalId] INTEGER NOT NULL,
[ClusterName] NVARCHAR(100),
[QQ] INTEGER NOT NULL,
[NickName] NVARCHAR(100),
[Message] TEXT,
[FontStyle] NVARCHAR(100),
[ReceiveTime] DATETIME
);");
                    }
                }
            }
            return base.Install();
        }

        public override bool Stop()
        {
            sdk = null;
            return base.Stop();
        }

        public override bool Start()
        {
            sdk = new QQClientSDK();
            sdk.ReceiveNormalIM += sdk_ReceiveNormalIM;
            sdk.ReceiveClusterIM += sdk_ReceiveClusterIM;
            sdk.ReceiveTempIM += sdk_ReceiveTempIM;
            sdk.ReceiveSubjectIM += sdk_ReceiveSubjectIM;

            return base.Start();
        }

        void sdk_ReceiveNormalIM(object sender, ReceiveNormalIMQQEventArgs e)
        {
            string FontStyle = string.Empty;
            if (e.Style != null)
                FontStyle = e.Style.ToString();
            var friend = client.GetFriend(e.QQ);
            lock (DbBase.LockDb)
            {
                using (DbBase db = new DbBase())
                {
                    db.Conn.Insert(new Messages() { Type = 0, ExternalId = 0, ClusterName = "", QQ = friend.QQ, NickName = friend.NickName, FontStyle = FontStyle, Message = e.Message, ReceiveTime = DateTime.Now });
                }
            }
        }

        void sdk_ReceiveClusterIM(object sender, ReceiveClusterIMQQEventArgs e)
        {
            string FontStyle = string.Empty;
            if (e.Style != null)
                FontStyle = e.Style.ToString();
            lock (DbBase.LockDb)
            {
                using (DbBase db = new DbBase())
                {
                    db.Conn.Insert(new Messages() { Type = 1, ExternalId = e.Cluster.ExternalId, ClusterName = e.Cluster.Name, QQ = e.ClusterMember.QQ, NickName = e.ClusterMember.Nick, FontStyle = FontStyle, Message = e.Message, ReceiveTime = DateTime.Now });
                }
            }
        }

        void sdk_ReceiveTempIM(object sender, ReceiveTempIMQQEventArgs e)
        {
            string FontStyle = string.Empty;
            if (e.Style != null)
                FontStyle = e.Style.ToString();
            var friend = client.GetFriend(e.QQ);
            lock (DbBase.LockDb)
            {
                using (DbBase db = new DbBase())
                {
                    db.Conn.Insert(new Messages() { Type = 2, ExternalId = 0, ClusterName = e.ExternalId.ToString(), QQ = friend.QQ, NickName = friend.NickName, FontStyle = FontStyle, Message = e.Message, ReceiveTime = DateTime.Now });
                }
            }
        }

        void sdk_ReceiveSubjectIM(object sender, ReceiveSubjectIMQQEventArgs e)
        {
            string FontStyle = string.Empty;
            if (e.Style != null)
                FontStyle = e.Style.ToString();
            var friend = client.GetFriend(e.QQ);
            lock (DbBase.LockDb)
            {
                using (DbBase db = new DbBase())
                {
                    db.Conn.Insert(new Messages() { Type = 3, ExternalId = e.ClusterId, ClusterName = "讨论组", QQ = friend.QQ, NickName = friend.NickName, FontStyle = FontStyle, Message = e.Message, ReceiveTime = DateTime.Now });
                }
            }
        }

        public override bool ShowForm()
        {
            new FormMain().ShowDialog();
            return true;
        }
    }
}
