﻿using QQPlus.Framework;
using QQPlus.Framework.Entities;
using QQPlus.Framework.SDK;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Web;

namespace QQPlus.OldWebPlugin
{
    public class WebPlugin : Plugin
    {
        public WebPlugin()
        {
            this.Id = "QQPlus.OldWebPlugin";
            this.PluginName = "旧版Web接口";
            this.Description = "机器人旧版Web接口";
            this.Author = "QQPlus";
            this.PluginId = 12;
        }

        public override bool ShowForm()
        {
            if (new FormMain(config).ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                StartGetWeb();
            }
            return base.ShowForm();
        }

        WebConfig config = null;
        public override bool Start()
        {
            config = PluginConfig.Init<WebConfig>();

            sdk = new QQClientSDK();
            sdk.ReceiveClusterIM += sdk_ReceiveClusterIM;
            sdk.ReceiveNormalIM += sdk_ReceiveNormalIM;
            sdk.ReceiveVibration += sdk_ReceiveVibration;
            sdk.SignatureChanged += sdk_SignatureChanged;

            StartGetWeb();

            return base.Start();
        }

        void sdk_SignatureChanged(object sender, SignatureChangedQQEventArgs e)
        {
            string ret = Post(-3, e.friend.QQ, e.friend.Signature.Sig);
            if (!string.IsNullOrWhiteSpace(ret))
            {
                client.SendMessage(e.friend.QQ, ret);
                e.Cancel = true;
            }
        }
        
        void sdk_ReceiveVibration(object sender, ReceiveVibrationQQEventArgs e)
        {
            string ret = Post(-6, e.QQ, string.Empty);
            if (!string.IsNullOrWhiteSpace(ret))
            {
                client.SendMessage(e.QQ, ret);
                e.Cancel = true;
            }
        }
        
        void sdk_ReceiveNormalIM(object sender, ReceiveNormalIMQQEventArgs e)
        {
            string ret = Post(0, e.QQ, e.Message);
            if (!string.IsNullOrWhiteSpace(ret))
            {
                client.SendMessage(e.QQ, ret);
                e.Cancel = true;
            }
        }

        void sdk_ReceiveClusterIM(object sender, ReceiveClusterIMQQEventArgs e)
        {
            var Message = e.Message.Replace("[image=", "[");
            string ret = Post(e.Cluster.ExternalId, e.ClusterMember.QQ, Message);
            if (!string.IsNullOrWhiteSpace(ret))
            {
                client.SendClusterMessage(e.Cluster.ExternalId, ret);
                e.Cancel = true;
            }
        }

        public override bool Stop()
        {
            StopGetWeb();
            OnLog("Web插件停止成功");
            sdk = null;
            return base.Stop();
        }

        string Post(long ExternalId, uint Sender, string Message, string other = "")
        {
            string result;
            try
            {
                string data = string.Format("key={0}&ExternalId={1}&Sender={2}&Message={3}&{4}", config.Key, ExternalId, Sender, System.Web.HttpUtility.UrlEncode(Message, Encoding.GetEncoding(config.Encode)), other);
                System.Net.ServicePointManager.DefaultConnectionLimit = 20;
                HttpWebRequest request = HttpWebRequest.Create(config.Url) as HttpWebRequest;
                request.ContentType = "application/x-www-form-urlencoded;charset=" + config.Encode;
                request.Method = "POST";
                byte[] B = Encoding.ASCII.GetBytes(data);
                request.ContentLength = B.Length;
                using (System.IO.Stream SW = request.GetRequestStream())
                {
                    SW.Write(B, 0, B.Length);
                    SW.Close();
                }
                HttpWebResponse response = request.GetResponse() as HttpWebResponse;
                StreamReader reader = new StreamReader(response.GetResponseStream(), Encoding.GetEncoding(config.Encode));
                result = reader.ReadToEnd();

                if (reader != null)
                    reader.Dispose();
                if (response != null)
                    response.Close();
            }
            catch
            {
                result = string.Empty;
            }

            if (!string.IsNullOrWhiteSpace(result) && result.Length > 5 && !result.Contains("</a>") && !result.Contains("</body>"))
                return result;
            return string.Empty;
        }

        /// <summary>
        /// 执行收到的指令，支持多个指令
        /// </summary>
        /// <param name="ret"></param>
        string ExecuteCommand(string ret)
        {
            try
            {
                if (!string.IsNullOrWhiteSpace(ret))
                {
                    if (!string.IsNullOrEmpty(ret) && ret.Length > 5 && !ret.Contains("</a>") && !ret.Contains("</body>"))
                    {
                        using (StringReader sr = new StringReader(ret))
                        {
                            long ExternalId = long.Parse(sr.ReadLine());
                            uint Sender = uint.Parse(sr.ReadLine());
                            string Message = sr.ReadToEnd();

                            if (ExternalId == 0)
                            {
                                client.SendMessage(Sender, Message);
                            }
                            else if (ExternalId > 0)
                            {
                                client.SendClusterMessage((uint)ExternalId, Message);
                            }
                            else if (ExternalId == -4)
                            {
                                client.ModifySignature(Message);
                            }
                            else if (ExternalId == -8)
                            {
                                //添加好友
                            }
                            else if (ExternalId == -12)
                            {
                                client.RemoveMember((uint)ExternalId, Sender);
                            }
                            else if (ExternalId == -13)
                            {
                                //加入黑名单
                            }
                            else if (ExternalId == -14)
                            {
                                client.JoinCluster(Sender, Message);
                            }
                            else if (ExternalId == -15)
                            {
                                //发送验证码
                            }
                            else if (ExternalId == -16)
                            {
                                client.ExitCluster(Sender);
                            }
                        }
                    }
                }
                return "ok";
            }
            catch (Exception ex)
            {
                OnLog("Web接口执行指令错误【这不是机器人的故障，请检查接口返回值】：{0}\n{1}", ex.Message, ex.StackTrace);
            }


            return "err";
        }

        Timer timerGetWeb;
        void StopGetWeb()
        {
            if (timerGetWeb != null)
                timerGetWeb.Dispose();
        }
        void StartGetWeb()
        {
            try
            {
                StopGetWeb();
                if (string.IsNullOrWhiteSpace(config.Url) || !config.Url.StartsWith("http://", StringComparison.CurrentCultureIgnoreCase) || string.IsNullOrWhiteSpace(config.Encode) || string.IsNullOrWhiteSpace(config.Key))
                    return;
                Encoding encoding = Encoding.GetEncoding(config.Encode);
                string geturl = config.Url.Contains('?') ? string.Format("{0}&key={1}", config.Url, config.Key) : string.Format("{0}?key={1}", config.Url, config.Key);


                timerGetWeb = new Timer(new TimerCallback((o) =>
                {
                    try
                    {
                        if (client.User.LoginStatus != LoginStatus.Login)
                            return;

                        OnLog("{0} 读取接口数据", DateTime.Now);
                        try
                        {
                            string result = encoding.GetString(new WebClient().DownloadData(geturl));
                            if (!string.IsNullOrWhiteSpace(result))
                            {
                                OnLog("读取到：{0}", result);
                                ExecuteCommand(result);
                            }
                        }
                        catch (Exception ex1)
                        {
                            OnLog("web接口请求失败：{0}\n{1}", ex1.Message, ex1.StackTrace);
                        }
                    }
                    catch (Exception ex)
                    {
                        OnLog("web接口请求失败：{0}\n{1}", ex.Message, ex.StackTrace);
                    }
                }), null, config.Sleep * 1000, config.Sleep * 1000);
            }
            catch (Exception ex)
            {
                OnLog("Web接口启动故障：{0}\n{1}", ex.Message, ex.StackTrace);
            }
        }
    }
}
