﻿using QQPlus.Framework.SDK;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using Dapper;
using System.Net;

namespace QQPlus.Org
{
    public class Class1 : Plugin
    {
        public OrgConfig config { get; set; }
        public Class1()
        {
            this.Id = "QQPlus.Org";
            this.PluginName = "上线通知";
            this.Description = "上线通知、订阅、取消订阅";
            DbBase.DataSource = string.Format("Data Source={0};Pooling=true;FailIfMissing=false", Path.Combine(PluginPath, "QQPlus.Org.db"));
        }

        public override bool Install()
        {
            lock (DbBase.LockDb)
            {
                using (DbBase db = new DbBase())
                {
                    Version v = new Version();
                    if (!db.Conn.TableExists("_history"))
                    {
                        //初始化
                        db.Conn.Execute(@"CREATE TABLE [_history] (
[Id] INTEGER  PRIMARY KEY NOT NULL,
[Ver] NVARCHAR(50)  NULL
);
INSERT INTO [_history] (ver) values('1.0.0.0');

CREATE TABLE [QQConfig] (
[Id] INTEGER PRIMARY KEY NOT NULL,
[QQ] INTEGER NOT NULL,
[Cmd1] NVARCHAR(50) NULL
);

");
                    }
                }
            }
            return base.Install();
        }

        public override bool Start()
        {
            LoadConfig();
            sdk = new QQClientSDK();
            sdk.AddMeFriend += sdk_AddMeFriend;
            sdk.ReceiveNormalIM += sdk_ReceiveNormalIM;
            sdk.StatusChanged += sdk_StatusChanged;
            return base.Start();
        }

        void sdk_StatusChanged(object sender, StatusChangedQQEventArgs e)
        {
            if (e.friend.QQStatus == Framework.QQStatus.在线)
            {
                var qc = Get(e.friend.QQ);
                if (qc != null)
                {
                    var ud = urls.FirstOrDefault(t => t.Cmd1 == qc.Cmd1);
                    if (ud != null)
                    {
                        Send(qc, ud);
                    }
                }
            }
        }

        void sdk_ReceiveNormalIM(object sender, ReceiveNormalIMQQEventArgs e)
        {
            var qc = Get(e.QQ);
            var ud = urls.FirstOrDefault(t => t.Cmd1.Equals(e.Message, StringComparison.InvariantCultureIgnoreCase));
            if (ud != null)
            {
                if (qc != null)
                {
                    qc.Cmd1 = ud.Cmd1;
                    Update(qc);
                }
                else
                {
                    qc = new QQConfig() { QQ = e.QQ, Cmd1 = ud.Cmd1 };
                    Add(qc);
                }
                client.SendMessage(e.QQ, ud.Txt1);
                e.Cancel = true;
                return;
            }
            ud = urls.FirstOrDefault(t => t.Cmd2.Equals(e.Message, StringComparison.InvariantCultureIgnoreCase));
            if (ud != null)
            {
                if (qc != null)
                {
                    Delete(e.QQ);
                    client.SendMessage(e.QQ, ud.Txt2);
                }
                else
                {
                    client.SendMessage(e.QQ, "请先订阅");
                }
                e.Cancel = true;
                return;
            }
        }

        void sdk_AddMeFriend(object sender, AddMeFriendQQEventArgs e)
        {
            if (urls.Count > 0)
            {
                var ud = urls[0];
                var qc = Get(e.QQ);
                if (qc == null)
                {
                    qc = new QQConfig() { QQ = e.QQ, Cmd1 = ud.Cmd1 };
                    Add(qc);
                }
                Send(qc, ud);
            }
        }

        private QQConfig Get(uint QQ)
        {
            lock (DbBase.LockDb)
            {
                using (DbBase db = new DbBase())
                {
                    return db.Conn.Query<QQConfig>("SELECT * FROM [QQConfig] WHERE QQ=@QQ LIMIT 1", new { QQ = QQ }).FirstOrDefault();
                }
            }
        }

        private void Add(QQConfig c)
        {
            lock (DbBase.LockDb)
            {
                using (DbBase db = new DbBase())
                {
                    db.Conn.Insert(c);
                }
            }
        }

        private void Update(QQConfig c)
        {
            lock (DbBase.LockDb)
            {
                using (DbBase db = new DbBase())
                {
                    db.Conn.Update(c);
                }
            }
        }

        private void Delete(uint QQ)
        {
            lock (DbBase.LockDb)
            {
                using (DbBase db = new DbBase())
                {
                    db.Conn.Execute("DELETE FROM QQConfig WHERE QQ=@QQ", new { QQ = QQ });
                }
            }
        }

        List<UrlData> urls = new List<UrlData>();
        string file = Path.Combine(PluginPath, "UrlData.txt");
        private void LoadConfig()
        {
            urls.Clear();
            if (!File.Exists(file))
                File.WriteAllText(file, "", Encoding.Default);
            string content = File.ReadAllText(file, Encoding.Default);
            var tmp = content.Split(new string[] { "\r\n\r\n" }, StringSplitOptions.RemoveEmptyEntries);
            tmp.ToList().ForEach(t =>
                {
                    var l = t.Split("\r\n".ToCharArray(), StringSplitOptions.RemoveEmptyEntries);
                    if (l != null && l.Length == 6)
                        urls.Add(new UrlData() { Cmd1 = l[0], Cmd2 = l[1], Encode = l[2], Url = l[3], Txt1 = l[4], Txt2 = l[5] });
                }
            );
            config = PluginConfig.Init<OrgConfig>();
        }

        List<QQTime> qts = new List<QQTime>();
        void Send(QQConfig qc, UrlData ud)
        {
            try
            {
                var now = DateTime.Now;
                qts.RemoveAll(t => t.Time < now);
                uint QQ = (uint)qc.QQ;
                if (qts.FirstOrDefault(t => t.QQ == QQ) == null)
                {
                    qts.Add(new QQTime() { QQ = QQ, Time = now.AddMinutes(config.Sleep) });
                    string body = Encoding.GetEncoding(ud.Encode).GetString(new WebClient().DownloadData(ud.Url));
                    client.SendMessage((uint)qc.QQ, body);
                    OnLog("给：{0} 发送：{1}", qc.QQ, body);
                }
                else
                    OnLog("Org插件过滤发送上线消息");
            }
            catch (Exception ex)
            {
                OnLog("发送消息故障：{0}", ex.Message);
            }
        }

        public override bool ShowForm()
        {
            new Form1(file,config).ShowDialog();
            LoadConfig();
            return base.ShowForm();
        }
    }

    class QQTime
    {
        public uint QQ { get; set; }
        /// <summary>
        /// 过期时间
        /// </summary>
        public DateTime Time { get; set; }
    }

    public class QQConfig
    {
        [Key]
        public long Id { get; set; }

        /// <summary>
        /// QQ号码
        /// </summary>
        public long QQ { get; set; }

        public string Cmd1 { get; set; }
    }

    public class OrgConfig : PluginConfig
    {
        public int Sleep { get; set; }

        public OrgConfig()
        {
            Sleep = 3;
        }
    }

    public class UrlData
    {
        /// <summary>
        /// 订阅
        /// </summary>
        public string Cmd1 { get; set; }

        public string Txt1 { get; set; }
        /// <summary>
        /// 取消订阅
        /// </summary>
        public string Cmd2 { get; set; }

        public string Txt2 { get; set; }
        /// <summary>
        /// 编码
        /// </summary>
        public string Encode { get; set; }
        /// <summary>
        /// 网址
        /// </summary>
        public string Url { get; set; }
    }
}
