﻿using QQPlus.Framework.SDK;
using QQPlus.Framework.Utils;
using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Threading.Tasks;

namespace QQPlus.Qianfu
{
    public class QianfuPlugin : Plugin
    {
        public QianfuPlugin()
        {
            this.Id = "QQPlus.QianfuPlugin";
            this.PluginName = "潜伏";
            this.Description = "记录群新人、群聊天记录、聊天内容匹配通知管理员";
            this.PluginId = 167;
        }

        /// <summary>
        /// 记录当天日期(最后发送日期）
        /// </summary>
        DateTime today = DateTime.MinValue;

        string path = null;

        public QianfuConfig config { get; set; }
        public override bool Start()
        {
            config = PluginConfig.Init<QianfuConfig>();
            StartSendEmail();

            sdk = new QQClientSDK();
            sdk.ReceiveClusterIM += sdk_ReceiveClusterIM;
            sdk.AddedToCluster += sdk_AddedToCluster;
            sdk.MemberExitCluster += sdk_MemberExitCluster;
            return base.Start();
        }

        void sdk_MemberExitCluster(object sender, MemberExitClusterQQEventArgs e)
        {
            string msg = e.ClusterMemberAdmin == null ? string.Format("{0}退出群：{1}", e.ClusterMember, e.Cluster) : string.Format("{0}被{1}踢出群：{2}", e.ClusterMember, e.ClusterMemberAdmin, e.Cluster);
            if (config.ExitCluster)
            {
                client.Config.AdminQQs.ForEach(q =>
                {
                    OnLog("给【{0}】发送消息：{1}", q, msg);
                    client.SendMessage(q, msg);
                }
                );
            }
        }

        void sdk_AddedToCluster(object sender, AddedToClusterQQEventArgs e)
        {
            if (config.JoinCluster)
            {
                string msg = string.Format("{0}加入群：{1}", e.ClusterMember, e.Cluster);
                client.Config.AdminQQs.ForEach(q =>
                    {
                        OnLog("给【{0}】发送消息：{1}", q, msg);
                        client.SendMessage(q, msg);
                    }
                );
            }
        }

        public override bool Stop()
        {
            if (timer != null)
                timer.Dispose();

            return base.Stop();
        }

        public override bool ShowForm()
        {
            new FormMain(this).ShowDialog();
            return base.ShowForm();
        }

        Timer timer;
        static readonly object obj = new object();
        private void WriteFile(string file, string content)
        {
            Task.Factory.StartNew(() =>
                {
                    try
                    {
                        lock (obj)
                        {
                            File.AppendAllText(file, content, Encoding.Default);
                        }
                    }
                    catch (Exception ex)
                    {
                        OnLog("潜伏插件写入文件异常：{0}\r\n{1}", ex.Message, ex.StackTrace);
                    }
                }
            );
        }

        /// <summary>
        /// 启动发送邮件线程
        /// </summary>
        private void StartSendEmail()
        {
            OnLog("启动发送邮件线程");
            var now = DateTime.Now;
            var next = DateTime.Now.AddDays(1);
            TimeSpan timespan = new DateTime(next.Year, next.Month, next.Day) - now;

            timer = new Timer((o) =>
                {
                    if (config.AllMessage)
                    {
                        OnLog("开始发送邮件");
                        if (!string.IsNullOrWhiteSpace(client.Config.SMTP) && !string.IsNullOrWhiteSpace(client.Config.AdminEmails))
                        {
                            QQPlus.Framework.Utils.Email.Send(client.Config.SMTP, client.Config.AdminEmails, string.Format("{0}群聊天记录", now.ToShortDateString()), "", Directory.GetFiles(path));
                        }
                    }
                }
            , null, (long)timespan.TotalMilliseconds, 24 * 60 * 60 * 1000);

        }

        void sdk_ReceiveClusterIM(object sender, ReceiveClusterIMQQEventArgs e)
        {
            var now = DateTime.Now;

            if (now.Year != today.Year || now.Month != today.Month || now.Day != today.Day)
            {
                //初始化path
                today = now;
                string tmp = Util.MapPath("群聊天记录");
                tmp = Path.Combine(tmp, today.ToString("yyyyMMdd"));
                path = Util.MapPath(tmp);
            }

            string file = Path.Combine(path, string.Format("{0}.txt", e.Cluster.ExternalId));

            string content = string.Format(@"{5} {0}({1})→{2}({3})
{4}
", e.Cluster.Name, e.Cluster.ExternalId, e.ClusterMember.Nick, e.ClusterMember.QQ, e.Message, now);

            WriteFile(file, content);

            if (config.Key)
            {
                if (Regex.IsMatch(e.Message, config.Pattern, RegexOptions.IgnoreCase))
                {
                    client.Config.AdminQQs.ForEach(q =>
                        {
                            client.SendMessage(q, content);
                            OnLog("给【{0}】发送消息：{1}", q, content);
                            Thread.Sleep(1000);
                        }
                    );
                }
            }
        }
    }
}
